
// dxh.hello.twi.c

// 270 HZ camera rate according to
// https://www.chenjingcheng.com/portfolio/indoor-tracking-using-ir-positioning-cam/

/* The Wii Remote includes a 128x96 monochrome camera with built-in
 * image processing. The camera looks through an infrared pass filter
 * in the remote's plastic casing. The camera's built-in image
 * processing is capable of tracking up to 4 moving objects, and these
 * data are the only data available to the host. Raw pixel data is not
 * available to the host, so the camera cannot be used to take a
 * conventional picture. The built-in processor uses 8x subpixel
 * analysis to provide 1024x768 resolution for the tracked points. */


#include <avr/io.h>
#include <util/delay.h>
#include <avr/pgmspace.h>

// library from https://github.com/g4lvanix/I2C-master-lib
// #include "i2c_master.h"

// library from https://github.com/alx741/avr_i2c
#include <i2cmaster.h>


#define output(directions,pin) (directions |= pin) // set port direction for output
#define set(port,pin) (port |= pin) // set port pin
#define clear(port,pin) (port &= (~pin)) // clear port pin
#define pin_test(pins,pin) (pins & pin) // test for port pin
#define bit_test(byte,bit) (byte & (1 << bit)) // test for bit set
//#define bit_delay_time 8.5 // bit delay for 115200 with overhead
#define bit_delay_time 102 // 9600 baud

#define bit_delay() _delay_us(bit_delay_time) // RS232 bit delay
#define half_bit_delay() _delay_us(bit_delay_time/2) // RS232 half bit delay
#define char_delay() _delay_ms(10) // char delay

#define serial_port PORTD
#define serial_direction DDRD
#define serial_pins PIND
// TX/RX from computer's perspective. in = computer sends TX. out = computer receives on RX.
#define serial_pin_in (1 << PD1)
#define serial_pin_out (1 << PD0)

#define max_buffer 25


#define led_port PORTC
#define led_direction DDRC
#define led_pin_1 (1 << PC1)
#define led_pin_2 (1 << PC0)
#define led_delay() _delay_ms(100) // LED delay


#define SCL_CLOCK  100000L // I2C clock in Hz
#define IR_CAMERA_ADDRESS 0xB0 // 8 bit address (0x58 << 1)



//
// SERIAL
//

  
void get_char(volatile unsigned char *pins, unsigned char pin, char *rxbyte) {
   //
   // read character into rxbyte on pins pin
   //    assumes line driver (inverts bits)
   //
   *rxbyte = 0;
   while (pin_test(*pins,pin))
      //
      // wait for start bit
      //
      ;
   //
   // delay to middle of first data bit
   //
   half_bit_delay();
   bit_delay();
   //
   // unrolled loop to read data bits
   //
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 0);
   else
      *rxbyte |= (0 << 0);
   bit_delay();
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 1);
   else
      *rxbyte |= (0 << 1);
   bit_delay();
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 2);
   else
      *rxbyte |= (0 << 2);
   bit_delay();
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 3);
   else
      *rxbyte |= (0 << 3);
   bit_delay();
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 4);
   else
      *rxbyte |= (0 << 4);
   bit_delay();
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 5);
   else
      *rxbyte |= (0 << 5);
   bit_delay();
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 6);
   else
      *rxbyte |= (0 << 6);
   bit_delay();
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 7);
   else
      *rxbyte |= (0 << 7);
   //
   // wait for stop bit
   //
   bit_delay();
   half_bit_delay();
   }

void put_char(volatile unsigned char *port, unsigned char pin, char txchar) {
   //
   // send character in txchar on port pin
   //    assumes line driver (inverts bits)
   //
   // start bit
   //
   clear(*port,pin);
   bit_delay();
   //
   // unrolled loop to write data bits
   //
   if bit_test(txchar,0)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,1)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,2)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,3)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,4)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,5)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,6)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,7)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   //
   // stop bit
   //
   set(*port,pin);
   bit_delay();
   //
   // char delay
   //
   bit_delay();
   }

void put_string(volatile unsigned char *port, unsigned char pin, char *str) {
   //
   // print a null-terminated string
   //
   static int index;
   index = 0;
   do {
      put_char(port, pin, str[index]);
      ++index;
      } while (str[index] != 0);
   }






/// --------- subroutines

void example_blink() {
  while(1) {
    set(led_port, led_pin_1);
    clear(led_port, led_pin_2);
    led_delay();
    clear(led_port, led_pin_1);
    set(led_port, led_pin_2);
    led_delay();
  }
  
}

void example_echo(void) {
  //
   // main
   //
   static char chr;
   static char buffer[max_buffer] = {0};
   static int index;
   //
   // set clock divider to /1
   //
   CLKPR = (1 << CLKPCE);
   CLKPR = (0 << CLKPS3) | (0 << CLKPS2) | (0 << CLKPS1) | (0 << CLKPS0);
   //
   // initialize output pins
   //
   set(serial_port, serial_pin_out);
   output(serial_direction, serial_pin_out);
   //
   // main loop
   //
   index = 0;
   while (1) {
      get_char(&serial_pins, serial_pin_in, &chr);
      put_string(&serial_port, serial_pin_out, "hello.ftdi.44.echo.c: you typed \"");
      buffer[index++] = chr;
      if (index == (max_buffer-1))
         index = 0;
      put_string(&serial_port, serial_pin_out, buffer);
      put_char(&serial_port, serial_pin_out, '\"');
      put_char(&serial_port, serial_pin_out, 10); // new line
      }
}










/* #define output(directions,pin) (directions |= pin) // set port direction for output */
/* #define set(port,pin) (port |= pin) // set port pin */
/* #define clear(port,pin) (port &= (~pin)) // clear port pin */
/* #define pin_test(pins,pin) (pins & pin) // test for port pin */
/* #define bit_test(byte,bit) (byte & (1 << bit)) // test for bit set */
/* //#define bit_delay_time 8.5 // bit delay for 115200 with overhead */
/* #define bit_delay_time 102 // 9600 baud */

/* #define bit_delay() _delay_us(bit_delay_time) // RS232 bit delay */
/* #define half_bit_delay() _delay_us(bit_delay_time/2) // RS232 half bit delay */
/* #define char_delay() _delay_ms(10) // char delay */

/* #define serial_port PORTD */
/* #define serial_direction DDRD */
/* #define serial_pins PIND */
/* #define serial_pin_in (1 << PD1) */
/* #define serial_pin_out (1 << PD0) */


/* #define max_buffer 25 */

/* //#define bit_delay_time 8.5 // bit delay for 115200 with overhead */
/* #define bit_delay_time 102 // 9600 */



/* // */
/* // TWI/I2C INTERFACE */
/* // */

/* /\* void I2C_init() { *\/ */
/* /\*   TWSR = 0;                         /\\* no prescaler *\\/ *\/ */
/* /\*   TWBR = ((F_CPU/SCL_CLOCK)-16)/2;  /\\* must be > 10 for stable operation *\\/ *\/ */
/* /\* } *\/ */

/* /\* char I2C_master_write_byte(unsigned char byte) { *\/ */
/* /\*   uint8_t   twst; *\/ */
/* /\*   // send data to the previously addressed device *\/ */
/* /\*   TWDR = byte; *\/ */
/* /\*   TWCR = (1<<TWINT) | (1<<TWEN); *\/ */
  
/* /\*   // wait until transmission completed *\/ */
/* /\*   while(!(TWCR & (1<<TWINT))); *\/ */
  
/* /\*   // check value of TWI Status Register. Mask prescaler bits *\/ */
/* /\*   twst = TW_STATUS & 0xF8; *\/ */
/* /\*   if( twst != TW_MT_DATA_ACK) return 1; *\/ */
/* /\*   return 0; *\/ */
/* /\* } *\/ */


/* /\* char I2C_master_write(unsigned char* data, unsigned char nbytes, unsigned char device_address) { *\/ */
/* /\*   // note: address should be 7 bits. *\/ */

/* /\*   unsigned char index,ret,device_address_write; *\/ */
/* /\*   uint8_t   twst; *\/ */
/* /\*   device_address_write = device_address << 1; *\/ */
  
/* /\*   // ---- START *\/ */
/* /\*   // send START condition *\/ */
/* /\*   TWCR = (1<<TWINT) | (1<<TWSTA) | (1<<TWEN); *\/ */

/* /\*   // wait until transmission completed *\/ */
/* /\*   while(!(TWCR & (1<<TWINT))); *\/ */

/* /\*   // check value of TWI Status Register. Mask prescaler bits. *\/ */
/* /\*   twst = TW_STATUS & 0xF8; *\/ */
/* /\*   if ( (twst != TW_START) && (twst != TW_REP_START)) continue; *\/ */

/* /\*   // ---- SEND ADDRESS *\/ */
  
/* /\*   // send device address *\/ */
/* /\*   TWDR = device_address_write; *\/ */
/* /\*   TWCR = (1<<TWINT) | (1<<TWEN); *\/ */

/* /\*   // wail until transmission completed *\/ */
/* /\*   while(!(TWCR & (1<<TWINT))); *\/ */


/* /\*   // ---- SEND DATA *\/ */
  
/* /\*   for (index = 0; index < nbytes; ++index) { *\/ */
/* /\*     ret = I2C_master_write_byte(data[index]); *\/ */
/* /\*     if (ret != 0) *\/ */
/* /\*       break; *\/ */
/* /\*     // *\/ */
/* /\*     // yes ACK, continue *\/ */
/* /\*       //	 *\/ */
/* /\*   } *\/ */


/* /\*   // ---- STOP *\/ */
  
/* /\*   /\\* send stop condition *\\/ *\/ */
/* /\*   TWCR = (1<<TWINT) | (1<<TWEN) | (1<<TWSTO); *\/ */
  
/* /\*   // wait until stop condition is executed and bus released *\/ */
/* /\*   while(TWCR & (1<<TWSTO)); *\/ */
  
 
/* /\*   return ret; *\/ */
  
/* /\* } *\/ */


/* // */
/* // SUBROUTINES  */
/* // */



int i2c_write_bytes(unsigned char* data, int numbytes) {
  unsigned char ret = 0, i;
  i2c_start_wait(IR_CAMERA_ADDRESS + I2C_WRITE);
  for(i=0; i<numbytes;++i) {
    ret |= i2c_write(data[i]);
  }
  i2c_stop();
  
    _delay_ms(100);
    _delay_ms(100);
    _delay_ms(100);


  return ret;
}

void twi_minimal_example() {
  // If successful, you should see
  // top light alone, bottom light alone, both lights.
  unsigned char i, led_on=0, error = 0;
  unsigned char data[17];

  //
  unsigned char p0 = 0xAA;
  unsigned char p1 = 0x64;
  unsigned char p2 = 0x63;
  unsigned char p3 = 0x03;
  
  
  i2c_init();
  
  clear(led_port, led_pin_1);
  clear(led_port, led_pin_2);

  
  _delay_ms(100); // essential delay for camera's microprocessor startup
  _delay_ms(100); // essential delay for camera's microprocessor startup
  _delay_ms(100); // essential delay for camera's microprocessor startup
  _delay_ms(100); // essential delay for camera's microprocessor startup
  _delay_ms(100); // essential delay for camera's microprocessor startup
  
  i2c_start_wait(IR_CAMERA_ADDRESS);
  set(led_port, led_pin_1);

  // start sensitivity settings
  error |= i2c_write(0x30);
  error |= i2c_write(0x01);
  _delay_ms(10);
  
  i2c_start_wait(IR_CAMERA_ADDRESS);
  error |= i2c_write(0x30);
  error |= i2c_write(0x08);
  _delay_ms(10);

  // sensitivity settings from http://www.kako.com/neta/2008-009/wii_remote_ir_sensor_sample2.zip
  // https://makezine.com/2008/11/22/hacking-the-wiimote-ir-ca/

  data[0] = 0;
  data[1] = 2;
  data[2] = 0;
  data[3] = 0;
  data[4] = 0x71;
  data[5] = 1;
  data[6] = 0;
  data[7] = p0;

  error |= i2c_write_bytes(data, 8);

  
  data[0] = 7;
  data[1] = 0;
  data[2] = p1;

  error |= i2c_write_bytes(data, 3);

  data[0] = 0x1A;
  data[1] = p2;
  data[2] = p3;
  
  error |= i2c_write_bytes(data, 3);

  

  
  /* // sensitivity settings */
  
  /* i2c_start_wait(IR_CAMERA_ADDRESS); */
  /* error |= i2c_write(0x06); */
  /* error |= i2c_write(0x90); */
  /* _delay_ms(10); */

  /* i2c_start_wait(IR_CAMERA_ADDRESS); */
  /* error |= i2c_write(0x08); */
  /* error |= i2c_write(0xC0); */
  /* _delay_ms(10); */
 
  /* i2c_start_wait(IR_CAMERA_ADDRESS); */
  /* error |= i2c_write(0x1A); */
  /* error |= i2c_write(0x40); */
  /* _delay_ms(10); */


  // finish sensitivity settings

  data[0] = 0x33;
  data[1] = 0x03;
  error |= i2c_write_bytes(data, 2);

  data[0] = 0x30;
  data[1] = 0x08;
  error |= i2c_write_bytes(data, 2);
  
  /* i2c_start_wait(IR_CAMERA_ADDRESS+I2C_WRITE); */
  /* error |= i2c_write(0x33); */
  /* error |= i2c_write(0x33); */
  /* _delay_ms(10); */


  if( error == 1 ) {
    /* set(led_port, led_pin_2); */
    clear(led_port, led_pin_1);
    clear(led_port, led_pin_2);
  }
  else {
    // success
    set(led_port, led_pin_2);
    clear(led_port, led_pin_1);

    i2c_start_wait(IR_CAMERA_ADDRESS+I2C_WRITE);
    error |= i2c_write(0x36); // request camera data
    i2c_stop();

    for(i=0;i<17;i++){data[i]=0;}
    
    i2c_start_wait(IR_CAMERA_ADDRESS+I2C_READ);
    data[0] = i2c_readAck();
    data[1] = i2c_readAck();
    data[2] = i2c_readAck();
    data[3] = i2c_readAck();
    data[4] = i2c_readAck();
    data[5] = i2c_readAck();
    data[6] = i2c_readAck();
    data[7] = i2c_readAck();
    data[8] = i2c_readAck();
    data[9] = i2c_readAck();
    data[10] = i2c_readAck();
    data[11] = i2c_readAck();
    data[12] = i2c_readAck();
    data[13] = i2c_readAck();
    data[14] = i2c_readAck();
    data[15] = i2c_readNak();
    data[16] = 0;
    
    
    set(led_port, led_pin_1);

    put_char(&serial_port, serial_pin_out, 1);
    put_char(&serial_port, serial_pin_out, 2);
    put_char(&serial_port, serial_pin_out, 3);
    put_char(&serial_port, serial_pin_out, 4);

    put_string(&serial_port, serial_pin_out, data);

    put_char(&serial_port, serial_pin_out, 5);
    put_char(&serial_port, serial_pin_out, 6);
    put_char(&serial_port, serial_pin_out, 7);
    put_char(&serial_port, serial_pin_out, 8);

    
    return;
    led_on = 1;
    while(1) {
      clear(led_port, led_pin_1);
      if(led_on == 1) {
	set(led_port, led_pin_1);
      }

      led_on = 1-led_on;
      
      
      i2c_stop();
    
    

      
      
      
    }
    
    
    /* while(1) { */
    /*   /\* clear(led_port, led_pin_1); *\/ */
    /*   /\* if(led_on) { *\/ */
    /*   /\* 	set(led_port, led_pin_1); *\/ */
    /*   /\* } *\/ */
      
    /*   /\* i2c_start_wait(IR_CAMERA_ADDRESS+I2C_READ); *\/ */
    /*   /\* set(led_port, led_pin_1); *\/ */
    /*   /\* i2c_stop(); *\/ */
    /*   /\* led_on = 0; *\/ */
    /*   _delay_ms(100); */
      
    /*   _delay_ms(100); */
      
    /*   _delay_ms(100); */
    /* } */
  }
}

/* void twi_camera_loop() { */
/*   unsigned char data[16]; */
/*   int error = 0;  */
/*   int led_on = 1; */
/*   int i; */

/*   // LED 1 (PC1) used to flag errors. */
/*   // LED 2 (PC0) toggled each time data is read. */
/*   clear(led_port, 1 << PC1); */
/*   clear(led_port, 1 << PC0); */


/*   _delay_ms(100); */
/*   i2c_start_wait(IR_CAMERA_ADDRESS+I2C_WRITE); */
/*   error |= i2c_write(0x36); // request camera data */
/*   i2c_stop(); */
    
  
/*   i2c_start_wait(IR_CAMERA_ADDRESS+I2C_READ);     */
/*   for(i=0;i<16;i++) { */

/*     data[i] = i2c_readAck(); */
    
/*   } */
/*   i2c_stop(); */
/*   led_on = 1-led_on; */
/* } */

/* void twi_camera_init() { */
/*   //unsigned char data[16]; */
/*   int error = 0;  */

/*   // sequence: */
/*   // 11 both lights on initially */
/*   // 00 both lights off after 1s delay */
/*   // 01 success initiating i2c */
/*   // 11 success sending first message */
/*   // 01 success sending all messages */

  
/*   set(led_port, led_pin_1); */
/*   set(led_port, led_pin_2); */
/*   i2c_init(); */
  
/*   _delay_ms(100); // essential delay for camera's microprocessor startup */
/*   _delay_ms(100); // essential delay for camera's microprocessor startup */
/*   _delay_ms(100); // essential delay for camera's microprocessor startup */
/*   _delay_ms(100); // essential delay for camera's microprocessor startup */
/*   _delay_ms(100); // essential delay for camera's microprocessor startup */
  
  
/*   clear(led_port, led_pin_1); */
/*   clear(led_port, led_pin_2); */

/*   i2c_start_wait(IR_CAMERA_ADDRESS);  */
/*   error |= i2c_write(0x30); */
/*   error |= i2c_write(0x01); */
/*   i2c_stop(); */
/*   _delay_ms(10); */
  
/*   i2c_start_wait(IR_CAMERA_ADDRESS);  */
/*   error |= i2c_write(0x30); */
/*   error |= i2c_write(0x08); */
/*   i2c_stop(); */
/*   _delay_ms(10); */
 
/*   i2c_start_wait(IR_CAMERA_ADDRESS);  */
/*   error |= i2c_write(0x06); */
/*   error |= i2c_write(0x90); */
/*   i2c_stop(); */
/*   _delay_ms(10); */

/*   i2c_start_wait(IR_CAMERA_ADDRESS);  */
/*   error |= i2c_write(0x08); */
/*   error |= i2c_write(0xC0); */
/*   i2c_stop(); */
/*   _delay_ms(10); */
 
/*   i2c_start_wait(IR_CAMERA_ADDRESS);  */
/*   error |= i2c_write(0x1A); */
/*   error |= i2c_write(0x40); */
/*   i2c_stop(); */
/*   _delay_ms(10); */
  
/*   i2c_start_wait(IR_CAMERA_ADDRESS);  */
/*   error |= i2c_write(0x33); */
/*   error |= i2c_write(0x33); */
/*   // i2c_stop(); */
/*   _delay_ms(10); */


/*   if( error == 1 ) { */
/*     set(led_port, led_pin_1); */
/*     clear(led_port, led_pin_2); */
/*   } */
/*   else { */
/*     set(led_port, led_pin_2); */
/*     clear(led_port, led_pin_1); */
/*     twi_camera_loop(); */
/*   } */

  
/*   return; */
  
/*   /\* put_char(serial_port, serial_pin_out, 'a'); *\/ */
  
  
/*   /\* if (i2c_start_wait(IR_CAMERA_ADDRESS) == 0)  { *\/ */
/*   /\*   // success *\/ */
/*   /\*     set(led_port, led_pin_1); *\/ */

/*   /\*   // Infrared sensor handshake/incantation.. *\/ */
/*   /\*  // *\/ */
/*   /\*  // This sequence sends (control_register_address, data) pairs to *\/ */
/*   /\*  // the computer onboard the infrared camera. Its purpose is to *\/ */
/*   /\*  // specify the sensitivity of the camera. *\/ */
/*   /\*  // *\/ */

/*   /\*   data[0] = 0x30; *\/ */
/*   /\*   data[1] = 0x01; *\/ */

    
/*   /\*   error |= i2c_transmit(IR_CAMERA_ADDRESS, data, 2); *\/ */


/*   /\*   if(error == 0) { *\/ */

/*   /\*     // successful first transmission *\/ */
/*   /\*     set(led_port, led_pin_2); *\/ */
/*   /\*   } *\/ */
    
    
/*   /\*   data[0] = 0x30; *\/ */
/*   /\*   data[1] = 0x08; *\/ */
/*   /\*   error |= i2c_transmit(IR_CAMERA_ADDRESS, data, 2); *\/ */
    
   
/*   /\*   data[0] = 0x06; *\/ */
/*   /\*   data[1] = 0x90; *\/ */
/*   /\*   error |= i2c_transmit(IR_CAMERA_ADDRESS, data, 2); *\/ */
   
/*   /\*   data[0] = 0x08; *\/ */
/*   /\*   data[1] = 0xC0; *\/ */
/*   /\*   error |= i2c_transmit(IR_CAMERA_ADDRESS, data, 2); *\/ */

/*   /\*   data[0] = 0x1A; *\/ */
/*   /\*   data[1] = 0x40; *\/ */
/*   /\*   error |= i2c_transmit(IR_CAMERA_ADDRESS, data, 2); *\/ */
   
/*   /\*   data[0] = 0x33; *\/ */
/*   /\*   data[1] = 0x33; *\/ */
/*   /\*   error |= i2c_transmit(IR_CAMERA_ADDRESS, data, 2); *\/ */
/*   /\*  //i2c_stop(); *\/ */

/*   /\*   if( error == 0) { *\/ */
/*   /\*     // all messages sent! 01 *\/ */
/*   /\*     clear(led_port, led_pin_1); *\/ */
/*   /\*     set(led_port, led_pin_2); *\/ */
/*   /\*   } *\/ */
/* } */


/* void example_twi() { */
/*   unsigned char data[16]; */
/*   int error = 0; */

/*   // sequence: */
/*   // 11 both lights on initially */
/*   // 00 both lights off after 1s delay */
/*   // 01 success initiating i2c */
/*   // 11 success sending first message */
/*   // 01 success sending all messages */
  
  
  
/*   /\* set(led_port, led_pin_1); *\/ */
/*   /\* clear(led_port, led_pin_2); *\/ */
  
/*   _delay_ms(100); */
/*   _delay_ms(100); */
/*   _delay_ms(100); */
/*   _delay_ms(100); */
/*   _delay_ms(100); */
/*   /\* _delay_ms(100); *\/ */
/*   /\* _delay_ms(100); *\/ */
/*   /\* _delay_ms(100); *\/ */
/*   /\* _delay_ms(100); *\/ */
/*   /\* _delay_ms(100); *\/ */
/*   /\* _delay_ms(100); *\/ */

/*   put_string(&serial_port, serial_pin_out, "abcd"); */

/*   // send START condition */

/*   // TWEN: enable twi */
/*   // TWSTA: desire to become master. must be cleared after start transmitted. */
/*   // TWINT: twi interrupt, pulls the clock down. must be cleared by writing 1. */
/*   TWCR =  (1 << TWEN) | (1<<TWINT) | (1<<TWSTA); */
 
/*   /\* while(1) { *\/ */
/*   /\*   put_string(&serial_port, serial_pin_out, "a"); *\/ */
/*   /\*   clear(led_port, led_pin_1);  *\/ */
/*   /\* } *\/ */
  
/*   /\* set(led_port, led_pin_1); *\/ */
/*   /\* set(led_port, led_pin_2); *\/ */
  
    
/*   /\*   set(led_port, led_pin_2); *\/ */
/*   /\*   // success *\/ */
/*   /\* } *\/ */
/*   /\* else { *\/ */
/*   /\*   set(led_port, led_pin_1); *\/ */
/*   /\* } *\/ */
  
  
  
 
/* } */

/* int main(void) { */
/*    // */
/*    // main */
/*    // */
/*    // set clock divider to /1 */
/*    // */
/*    CLKPR = (1 << CLKPCE); */
/*    CLKPR = (0 << CLKPS3) | (0 << CLKPS2) | (0 << CLKPS1) | (0 << CLKPS0); */
   
/*    // */
/*    // initialize output pins */
/*    // */
/*    set(serial_port, serial_pin_out); */
/*    output(serial_direction, serial_pin_out); */

/*    /\* clear(led_port, led_pin_1); *\/ */
/*    /\* clear(led_port, led_pin_2); *\/ */
/*    /\* output(led_direction, led_pin_1); *\/ */
/*    /\* output(led_direction, led_pin_2); *\/ */
/*    /\* set(led_port, led_pin_1);  *\/ */
/*    /\* set(led_port, led_pin_2);  *\/ */

/*    output(led_direction, 1 << PC0); */
/*    output(led_direction, 1 << PC1); */

/*    example_echo(); */
/*    //put_char(&serial_port, serial_pin_out, 'x'); */

/*    //twi_camera_init(); */
/*    //   twi_minimal_example(); */
   
/*    //example_twi(); */
/*    /\* while(1) { *\/ */
/*    /\*   put_string(&serial_port, serial_pin_out, "a"); *\/ */
/*    /\* } *\/ */
   
/*    //  example_blink(); */
/* } */


int main(void) {
  CLKPR = (1 << CLKPCE);
  CLKPR = (0 << CLKPS3) | (0 << CLKPS2) | (0 << CLKPS1) | (0 << CLKPS0);
   
   //
   // initialize output pins
   //
   set(serial_port, serial_pin_out);
   output(serial_direction, serial_pin_out);

   output(led_direction, led_pin_1);
   output(led_direction, led_pin_2);
   clear(led_port, led_pin_1);
   clear(led_port, led_pin_2);

   output(led_direction, (1 << PC0));
   output(led_direction, (1 << PC1));
   output(led_direction, (1 << PC2));

   set(led_port, (1 << PC0));
   set(led_port, (1 << PC1));
   set(led_port, (1 << PC2));

   /* while(1) { */
   /*   put_char(&serial_port, serial_pin_out, 'a'); */
   /* } */
   //   example_blink();
   //example_echo();
   twi_minimal_example();
}
